package main

import (
	"bytes"
	"image/png"
	"os"

	"bitbucket.org/nimblebet/raystuff/internal/asset"
	"bitbucket.org/nimblebet/raystuff/internal/enum"
	"bitbucket.org/nimblebet/raystuff/internal/game"
	rl "github.com/gen2brain/raylib-go/raylib"
)

func loadTexture(data []byte) rl.Texture2D {
	raw, err := png.Decode(bytes.NewReader(data))
	if err != nil {
		return rl.Texture2D{}
	}

	img := rl.NewImageFromImage(raw)

	tex := rl.LoadTextureFromImage(img)

	rl.UnloadImage(img)

	return tex
}

func loadTextureFromFile(path string) rl.Texture2D {
	f, err := os.Open(path)
	if err != nil {
		return rl.Texture2D{}
	}

	raw, err := png.Decode(f)
	if err != nil {
		return rl.Texture2D{}
	}

	img := rl.NewImageFromImage(raw)

	tex := rl.LoadTextureFromImage(img)

	rl.UnloadImage(img)

	return tex
}

func unloadTextures(c game.Config) {
	rl.UnloadTexture(c.Textures.Botles.Blue)
	rl.UnloadTexture(c.Textures.Botles.Green)
	rl.UnloadTexture(c.Textures.Botles.Orange)
	rl.UnloadTexture(c.Textures.Botles.Pink)
	rl.UnloadTexture(c.Textures.TaskWindow)
	rl.UnloadTexture(c.Textures.Fact)
	rl.UnloadTexture(c.Textures.GrownPlant)
	rl.UnloadTexture(c.Textures.LoseBackground)
	rl.UnloadTexture(c.Textures.MainBottleEmpty)
	rl.UnloadTexture(c.Textures.MainBottleFull)
	rl.UnloadTexture(c.Textures.PlantActive)
	rl.UnloadTexture(c.Textures.PlantInactive)
	rl.UnloadTexture(c.Textures.GardenBackground)
}

func initGameConfig() (game.Config, error) {
	c := game.Config{
		TileSize:   32,
		MapSize:    25,
		LabMapSize: rl.NewVector2(9, 5),
		Zoom:       2,
		Garden: game.GardenConfig{
			CellSize:     100,
			ColumnsCount: 4,
			RowsCount:    2,
		},
		Composition: game.ClassComposition{
			Family: map[enum.PlantFamily]int{
				enum.Fabales:     20,
				enum.Solanales:   10,
				enum.Beta:        50,
				enum.Cannabaceae: 30,
			},
			Height: map[enum.PlantHeight]int{
				enum.LowPlantHeight:    20,
				enum.MediumPlantHeight: 10,
				enum.HighPlantHeight:   30,
			},
			CalorificValue: map[enum.PlantCalorificValue]int{
				enum.LowCalorificValue:    20,
				enum.MediumCalorificValue: 10,
				enum.HighCalorificValue:   30,
			},
		},
		Plants: []game.Plant{
			game.Plant{
				Name: "Batata",
				Description: `Sweet potatoes have been grown and eaten
for a very long time. In fact, it is
said that the sweet potato is the
oldest root vegetable in the world.
It has been grown and eaten for at
least 5,000 years`,
				Family:         enum.Solanales,
				Height:         enum.MediumPlantHeight,
				CalorificValue: enum.MediumCalorificValue,
			},
			game.Plant{
				Name: "Potato",
				Description: `The potato is the most important 
non-cereal crop in the world, and 
fourth most important crop overall.`,
				Family:         enum.Solanales,
				Height:         enum.LowPlantHeight,
				CalorificValue: enum.MediumCalorificValue,
			},
			game.Plant{
				Name: "Tomato",
				Description: `Because the tomato has seeds and 
grows from a flowering plant 
botanically it is classed as a fruit
not a vegetable.`,
				Family:         enum.Solanales,
				Height:         enum.MediumPlantHeight,
				CalorificValue: enum.LowCalorificValue,
			},
			game.Plant{
				Name: "Pea",
				Description: `The oldest pea was found in 
Thailand. It was 3000 years old!`,
				Family:         enum.Fabales,
				Height:         enum.MediumPlantHeight,
				CalorificValue: enum.HighCalorificValue,
			},
			game.Plant{
				Name: "Corn",
				Description: `Maize is a native Taino word 
meaning “sacred mother,” or 
“giver of life.”`,
				Family:         enum.Fabales,
				Height:         enum.HighPlantHeight,
				CalorificValue: enum.HighCalorificValue,
			},
			game.Plant{
				Name: "Cannabis",
				Description: `Because it’s a fast-growing plant 
that’s easy to cultivate and has many 
uses, hemp was widely grown throughout 
colonial America and at  Spanish missions
in the Southwest. In the early 1600s,
the Virginia, Massachusetts and Connecticut 
colonies required farmers to grow hemp.`,
				Family:         enum.Cannabaceae,
				Height:         enum.HighPlantHeight,
				CalorificValue: enum.LowCalorificValue,
			},
			game.Plant{
				Name: "Beetroot",
				Description: `Record breakers: The world's
heaviest beetroot weighed 
23.4 kg (51.48 lb.) and was
grown by Ian Neale from Somerset 
in 2001.`,
				Family:         enum.Beta,
				Height:         enum.LowPlantHeight,
				CalorificValue: enum.MediumCalorificValue,
			},
		},
	}

	rl.InitWindow(c.TileSize*int32(c.MapSize), c.TileSize*int32(c.MapSize), "Repair")

	rl.SetTargetFPS(60)

	c.Textures.TaskWindow = loadTexture(asset.TaskWindow)
	c.Textures.Fact = loadTexture(asset.Fact)
	c.Textures.GrownPlant = loadTexture(asset.GrownPlant)
	c.Textures.LoseBackground = loadTexture(asset.LoseBackground)
	c.Textures.WinBackground = loadTexture(asset.BgWin)
	c.Textures.MainBottleFull = loadTexture(asset.MainBottleFull)
	c.Textures.MainBottleEmpty = loadTexture(asset.MainBottleEmpty)
	c.Textures.PlantActive = loadTexture(asset.PlantActive)
	c.Textures.PlantInactive = loadTexture(asset.PlantInactive)
	c.Textures.GardenBackground = loadTexture(asset.GardenBackground)

	c.Textures.Botles.Blue = loadTexture(asset.BottleBlue)
	c.Textures.Botles.Green = loadTexture(asset.BottleGreen)
	c.Textures.Botles.Orange = loadTexture(asset.BottleOrange)
	c.Textures.Botles.Pink = loadTexture(asset.BottlePink)
	c.Textures.BotlesRand = []rl.Texture2D{
		c.Textures.Botles.Green,
		c.Textures.Botles.Orange,
		c.Textures.Botles.Pink,
	}

	c.Textures.City = loadTexture(asset.City)
	c.Textures.Lab = loadTexture(asset.Lab)

	c.FontRomulus = rl.LoadFont("assets/romulus.png")

	return c, nil
}
